/*
 * IRC - Internet Relay Chat, ircd/s_socks.c
 * Copyright (C) 1992 Darren Reed
 * Copyright (C) 1999 IRC-Hispano.org - ESNET - jcea & savage
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "sys.h"
#include <sys/socket.h>
#if HAVE_SYS_FILE_H
#include <sys/file.h>
#endif
#if HAVE_SYS_IOCTL_H
#include <sys/ioctl.h>
#endif
#ifdef	UNIXPORT
#include <sys/un.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HPUX
#include <arpa/inet.h>
#endif /* HPUX */
#if HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef USE_SYSLOG
#include <syslog.h>
#endif
#include <pwd.h>
#include <sys/types.h>

#include <assert.h>

#include "h.h"
#include "res.h"
#include "struct.h"
#include "common.h"
#include "send.h"
#include "s_bsd.h"
#include "s_misc.h"
#include "support.h"
#include "ircd.h"
#include "s_socks.h"
#include "sprintf_irc.h"
#include "hash.h"

#ifdef PROXY_PROTECTION
#include "IPcheck.h"
#include "s_bdd.h"
#endif

#ifdef PROXY_PROTECTION

RCSTAG_CC("$Id: s_socks.c,v 1.1.1.1 2003/05/11 22:08:59 obiwansp Exp $");

#define PROXY_START ":%s NOTICE SOCKS4 :*** Checking WinGate/SocksProxy security...\r\n"
#define PROXY_PASS ":%s NOTICE SOCKS4 :*** Proxy test passed.\r\n"
#define PROXY_FAIL ":%s NOTICE SOCKS4 :*** Proxy is UNSECURE.\r\n"
#define PROXY_PASS_CACHE ":%s NOTICE SOCKS4 :*** Proxy test passed (CACHED! - TTL: %d seconds).\r\n"
#define PROXY_FAIL_CACHE ":%s NOTICE SOCKS4 :*** Proxy is UNSECURE (CACHED! - TTL: %d seconds).\r\n"




static void avisa_socks(aClient *cptr)
{
  aChannel *chptr;
  char *host, *hostip;
  struct db_reg *reg;

/* ?Acabamos de informar? */
  if (IPcheck_proxy_cache_get(cptr, NULL))
    return;

  reg = db_buscar_registro(BDD_BOTSDB, BDD_CANAL_NOTIF_SOCKS);
  if (!reg)
    return;

  if ((chptr = FindChannel(reg->valor)))
  {
    hostip = inetntoa(cptr->ip);
    if (cptr->hostp)
      host = cptr->hostp->h_name;
    else
      host = hostip;

    sendto_channel_butone(&me, &me, chptr,
	":%s PRIVMSG %s :Proxy SOCKS inseguro en %s (%s)", me.name, reg->valor,
	host, hostip);
  }
}

/*
 * start_socks
 *
 * Flag the client to show that an attempt to contact the socks server on
 * the client's host.  The connect and subsequently the socket are all put
 * into 'non-blocking' mode.
 */
void start_socks(aClient *cptr)
{
  struct sockaddr_in sock;
  int err;
  char text[1024];
  int cache;
  int caducidad;

  if ((cache = IPcheck_proxy_cache_get(cptr, &caducidad)))
  {
    if (cache == 1)
    {
      sprintf_irc(text, PROXY_PASS_CACHE, me.name, caducidad);
      write(cptr->fd, text, strlen(text));
      if (!DoingDNS(cptr) && !DoingAuth(cptr))
	SetAccess(cptr);
      return;
    }
    else
    {
      sprintf_irc(text, PROXY_FAIL_CACHE, me.name, caducidad);
      write(cptr->fd, text, strlen(text));

      /* close connection */
      close(cptr->socksfd);
      close(cptr->authfd);
      exit_client(cptr, cptr, &me, "Unsecure SocksProxy or WinGate");
      return;
    }
  }

  Debug((DEBUG_NOTICE, "start_socks(%p) fd %d status %d",
      cptr, cptr->fd, cptr->status));

  alarm(2);
  cptr->socksfd = socket(AF_INET, SOCK_STREAM, 0);
  err = errno;
  alarm(0);
  if (cptr->socksfd < 0 && err == EAGAIN)
    sendto_ops("Can't allocate fd for socks on %s : socket: No more sockets",
	get_client_name(cptr, TRUE));

  if (cptr->socksfd < 0)
  {
#ifdef	USE_SYSLOG
    syslog(LOG_ERR, "Unable to create socks socket for %s:%m",
	get_client_name(cptr, TRUE));
#endif
    Debug((DEBUG_ERROR, "Unable to create socks socket for %s:%s",
	get_client_name(cptr, TRUE), strerror(get_sockerr(cptr))));
    if (!DoingDNS(cptr) && !DoingAuth(cptr))
      SetAccess(cptr);
    ircstp->is_abad++;
    return;
  }
  if (cptr->socksfd >= (MAXCONNECTIONS - 2))
  {
    sendto_ops("Can't allocate fd for socks on %s", get_client_name(cptr,
	TRUE));
    close(cptr->socksfd);
    return;
  }

  set_non_blocking(cptr->socksfd, cptr);

#ifdef VIRTUAL_HOST
  if (bind(cptr->socksfd, (struct sockaddr *)&vserv, sizeof(vserv)) == -1)
  {
    report_error("binding socks stream socket %s: %s", cptr);
    close(cptr->fd);
    return;
  }
#endif
  memcpy(&sock.sin_addr, &cptr->ip, sizeof(struct in_addr));

  sock.sin_port = htons(1080);
  sock.sin_family = AF_INET;
  sprintf_irc(text, PROXY_START, me.name);
  write(cptr->fd, text, strlen(text));
  alarm((unsigned)4);
  if (connect(cptr->socksfd, (struct sockaddr *)&sock,
      sizeof(sock)) == -1 && errno != EINPROGRESS)
  {
    ircstp->is_abad++;
    /*
     * No error report from this...
     */
    alarm((unsigned)0);
    close(cptr->socksfd);
    cptr->socksfd = -1;
    sprintf_irc(text, PROXY_PASS, me.name);
    write(cptr->fd, text, strlen(text));

    IPcheck_proxy_cache_set(cptr, 1);

    if (!DoingDNS(cptr) && !DoingAuth(cptr))
      SetAccess(cptr);
    return;
  }
  alarm((unsigned)0);
  SetSocks(cptr);
  SetWrSocks(cptr);
  if (cptr->socksfd > highest_fd)
    highest_fd = cptr->socksfd;
  return;
}

/*
 * send_socksports
 *
 */
void send_socksports(aClient *cptr)
{
  struct sockaddr_in us, them;
  char socksbuf[255], *uname;
  size_t ulen, tlen;
  struct socks4 *sk;
  char text[1024];

  Debug((DEBUG_NOTICE, "write_socksports(%p) fd %d socksfd %d stat %d",
      cptr, cptr->fd, cptr->socksfd, cptr->status));
  tlen = ulen = sizeof(us);
  if (getsockname(cptr->fd, (struct sockaddr *)&us, &ulen) ||
      getpeername(cptr->fd, (struct sockaddr *)&them, &tlen))
  {
#ifdef	USE_SYSLOG
    syslog(LOG_ERR, "socks get{sock,peer}name error for %s:%m",
	get_client_name(cptr, TRUE));
#endif
    goto sockssenderr;
  }

  sk = (struct socks4 *)socksbuf;
  uname = socksbuf + sizeof(struct socks4);

  sk->vn = 4;
  sk->cd = 1;
  sk->dstport = us.sin_port;
  sk->dstip = us.sin_addr.s_addr;
/* Ningun nombre de usuario */
  *uname = '\0';

  Debug((DEBUG_SEND, "sending connection request to socks port %s.1080",
      inetntoa(them.sin_addr)));
  if (write(cptr->socksfd, socksbuf, sizeof(struct socks4) + strlen(uname) + 1)
      != (int)(sizeof(struct socks4) + strlen(uname) + 1))
  {
  sockssenderr:
    sprintf_irc(text, PROXY_PASS, me.name);
    write(cptr->fd, text, strlen(text));

    IPcheck_proxy_cache_set(cptr, 1);

    close(cptr->socksfd);
    if (cptr->socksfd == highest_fd)
      while (!loc_clients[highest_fd])
	highest_fd--;
    cptr->socksfd = -1;
    ClearSocks(cptr);
    if (!DoingDNS(cptr) && !DoingAuth(cptr))
      SetAccess(cptr);
  }
  ClearWrSocks(cptr);
  return;
}

/*
 * read_socksports
 *
 * read the reply (if any) from the ident server we connected to.
 * The actual read processijng here is pretty weak - no handling of the reply
 * if it is fragmented by IP.
 */
void read_socksports(aClient *cptr)
{
  struct socks4 *sk;
  char socksbuf[255];
  char text[1024];

  sk = (struct socks4 *)socksbuf;

  Debug((DEBUG_NOTICE, "read_socksports(%p) fd %d socksfd %d stat %d",
      cptr, cptr->fd, cptr->socksfd, cptr->status));

  if (cptr->socksfd > 0
      && read(cptr->socksfd, socksbuf,
      sizeof(struct socks4)) == sizeof(struct socks4))
  {
    switch (sk->cd)
    {
      case 90:
	sprintf_irc(text, ":%s NOTICE SOCKS4 :*** Proxy is UNSECURE.\r\n",
	    me.name);
	write(cptr->fd, text, strlen(text));

	avisa_socks(cptr);

	IPcheck_proxy_cache_set(cptr, 2);

	/* close connection */
	close(cptr->socksfd);
	close(cptr->authfd);
	exit_client(cptr, cptr, &me, "Unsecure SocksProxy or WinGate");
	return;
      default:
	break;
    }
  }

  sprintf_irc(text, PROXY_PASS, me.name);
  write(cptr->fd, text, strlen(text));

  IPcheck_proxy_cache_set(cptr, 1);

  cptr->lasttime = now;
  close(cptr->socksfd);
  if (cptr->socksfd == highest_fd)
    while (!loc_clients[highest_fd])
      highest_fd--;
  cptr->socksfd = -1;
  ClearSocks(cptr);
  ClearWrSocks(cptr);
  if (!DoingDNS(cptr) && !DoingAuth(cptr))
    SetAccess(cptr);
  return;
}

#endif
